
<#

    .SYNOPSIS


    .DESCRIPTION
    To fix https://support.microsoft.com/en-us/help/3213242/software-metering-agent-fails-with-software-metering-failed-to-start-p
    .PARAMETER

    .EXAMPLE

    .NOTES
    Author: Morten Rønborg
    Date: 12-09-2018
    Last Updated: 12-09-2018

#>

################################################
Function Write-Log
{
    param 
    (
        [Parameter(Mandatory=$true, HelpMessage="Provide a message")][string]$LogOutput,
        [Parameter(Mandatory=$true, HelpMessage="Provide the function name")][string]$FunctionName,
        [Parameter(Mandatory=$false, HelpMessage="Provide the scriptlinenumber")][string]$ScriptLine,
        [Parameter(Mandatory=$false, HelpMessage="Provide path, default is .\Logs")][string]$Path,
        [Parameter(Mandatory=$false, HelpMessage="Provide name for the logs")][string]$Name,
        [Parameter(Mandatory=$false, HelpMessage="Provide level, 1 = default, 2 = warning 3 = error")][ValidateSet(1, 2, 3)][int]$LogLevel = 1
    )

    #If the scriptline is not defined then use from the invocation
    If(!($ScriptLine)){
        $ScriptLine = $($MyInvocation.ScriptLineNumber)
    }

    if($LogOutput){

        #Date for the lognaming
        $FullLogName = ($Path + "\" + $Name + ".log")
        $FullSecodaryLogName = ($FullLogName).Replace(".log",".lo_")

        #If the log has reached over 20 mb then rename it
        if(Test-Path $FullLogName){
            if((Get-Item $FullLogName).Length -gt 5000kb){
                if(Test-Path $FullSecodaryLogName){
                    Remove-Item -Path $FullSecodaryLogName -force
                }
                Rename-Item -Path $FullLogName -NewName $FullSecodaryLogName
            }
        }

        #First check if folder/logfile excist, if not then create it
        if(!(test-path $Path)){
            New-Item -ItemType Directory -Force -Path $Path -ErrorAction SilentlyContinue
        }

        #Get current date and time to write to log
        $TimeGenerated = "$(Get-Date -Format HH:mm:ss).$((Get-Date).Millisecond)+000"

        #Construct the logline format
        $Line = '<![LOG[{0}]LOG]!><time="{1}" date="{2}" component="{3}" context="" type="{4}" thread="" file="">'

        #Define line
        $LineFormat = $logOutput, $TimeGenerated, (Get-Date -Format MM-dd-yyyy), "$($FunctionName):$($Scriptline)", $LogLevel

        #Append line
        $Line = $Line -f $LineFormat

        #Write log
        try {
            Write-Host ("[$($FunctionName):$($Scriptline)]" + $logOutput)
            $Line | Out-File -FilePath ($Path + "\" + $Name + ".log") -Append -NoClobber -Force -Encoding 'UTF8' -ErrorAction 'Stop'
        }
        catch {
            Write-Host "$_"
        }
    }
}

# logging variables
$LogLocation = "$env:windir\Logs\Software"
$LogName = "Fix-SCCMAgentSoftwareMeteringAgent"
$CCMAgentLogFolder = "$env:windir\CCM\Logs"
$MeteringLog = "mtrmgr.log"
$DetectionError = "StartPrepDriver - OpenService Failed with error"
$Elevated = (New-Object Security.Principal.WindowsPrincipal([Security.Principal.WindowsIdentity]::GetCurrent())).IsInRole([Security.Principal.WindowsBuiltInRole]::Administrator)

Write-Log -LogOutput ("*********************************************** SCRIPT START ***********************************************") -FunctionName $($MyInvocation.MyCommand)-Path $LogLocation -Name $LogName | Out-Null
Write-log -LogOutput ("Running in the context of '$($env:username)'") -FunctionName $($MyInvocation.MyCommand)-Path $LogLocation -Name $LogName | Out-Null
Write-log -LogOutput ("Running in elevated mode = $($Elevated)") -FunctionName $($MyInvocation.MyCommand)-Path $LogLocation -Name $LogName | Out-Null

#Check if the error is present
if (Get-Content -Path "$($CCMAgentLogFolder)\$($MeteringLog)" | Select-String $DetectionError) {

    #Fix the client
    Write-Log -LogOutput ("'$($DetectionError)' found in the '$($CCMAgentLogFolder)\$($MeteringLog)' log, loading the PrepDriver and restarting the SMS Agent Host..") -FunctionName $($MyInvocation.MyCommand)-Path $LogLocation -Name $LogName | Out-Null
    try {
        Start-Process -FilePath "RUNDLL32.EXE" -ArgumentList "SETUPAPI.DLL,InstallHinfSection DefaultInstall 128 C:\WINDOWS\CCM\prepdrv.inf"
        Restart-Service -Name "SMS Agent Host" -Force
    }
    catch {
        Write-Log -LogOutput ("$_") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName -LogLevel 3 | Out-Null
    }

}else {
    Write-Log -LogOutput ("'$($DetectionError)' NOT found in the '$($CCMAgentLogFolder)\$($MeteringLog)' log, skipping fix.") -FunctionName $($MyInvocation.MyCommand)-Path $LogLocation -Name $LogName | Out-Null
}

Write-Log -LogOutput ("*********************************************** SCRIPT END ***********************************************") -FunctionName $($MyInvocation.MyCommand)-Path $LogLocation -Name $LogName | Out-Null
